import anonation.Varchar;

import java.io.File;
import java.lang.reflect.Field;
import java.sql.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * SqliteClient
 *
 * @author wangbing
 */
public class SqliteClient {

    private Connection connection;
    private Statement statement;
    private ResultSet resultSet;
    private String dbFilePath;
    private Map<String, Class> classMap = new HashMap<>();

    /**
     * 构造函数
     *
     * @param dbFile    文件
     * @param classList 注册对象
     * @throws ClassNotFoundException
     * @throws SQLException
     */
    public SqliteClient(File dbFile, List<Class> classList) throws ClassNotFoundException, SQLException {
        this.dbFilePath = dbFile.getAbsolutePath();
        for (Class aClass : classList) {
            classMap.put(aClass.getName(), aClass);
        }
        if (!dbFile.exists()) {
            connection = getConnection();
            for (String key : classMap.keySet()) {
                Class object = classMap.get(key);
                StringBuffer sql = new StringBuffer();
                String name = object.getSimpleName();

                sql.append("CREATE TABLE ");
                sql.append(name);
                sql.append(" (");

                Field[] fields = object.getDeclaredFields();
                for (Field f : fields) {
                    if (f.isAnnotationPresent(Varchar.class)) {
                        Varchar bind = f.getAnnotation(Varchar.class);
                        int length = bind.value();
                        sql.append(f.getName().toUpperCase());
                        sql.append(" VARCHAR(" + length + "),");
                    }
                }

                sql.replace(sql.length() - 1, sql.length(), "");
                sql.append(")");
                System.out.println("SQL ==> " + sql.toString());
                execute(sql.toString());
            }
        }
    }

    public <T> void insert(Class<T> poClass, T po) throws SQLException, ClassNotFoundException {
        try {
            Class aClass = classMap.get(poClass.getName());
            if (aClass == null) {
                System.err.println(poClass.getName() + " not found.");
            } else {
                StringBuffer sql = new StringBuffer();
                sql.append("INSERT INTO ");
                sql.append(aClass.getSimpleName().toUpperCase());
                sql.append("(");

                //获取字段列表
                List<Field> fs = new ArrayList<>();
                for (Field f : poClass.getDeclaredFields()) {
                    if (f.isAnnotationPresent(Varchar.class)) {
                        f.setAccessible(true);
                        fs.add(f);
                    }
                }

                StringBuffer fieldsSql = new StringBuffer();
                StringBuffer valueSql = new StringBuffer();

                for (int i = 0; i < fs.size(); i++) {
                    Field f = fs.get(i);
                    fieldsSql.append(f.getName().toUpperCase());

                    Object value = f.get(po);
                    if (value instanceof String) {
                        valueSql.append("'");
                        valueSql.append(value);
                        valueSql.append("'");
                    } else {
                        valueSql.append(value);
                    }
                    if (i != fs.size() - 1) {
                        fieldsSql.append(",");
                        valueSql.append(",");
                    }
                }

                sql.append(fieldsSql);
                sql.append(") VALUES (");
                sql.append(valueSql);
                sql.append(")");
                System.out.println("SQL ==> " + sql.toString());
                executeUpdate(sql.toString());
            }
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } finally {
            destroyed();
        }
    }

    public <T> int delete(Class<T> poClass, String... wheres) throws SQLException, ClassNotFoundException {
        try {
            Class aClass = classMap.get(poClass.getName());
            if (aClass == null) {
                System.err.println(poClass.getName() + " not found.");
            } else {
                StringBuffer sql = new StringBuffer();
                sql.append("DELETE FROM ");
                sql.append(aClass.getSimpleName().toUpperCase());

                if (wheres.length > 0) {
                    sql.append(" WHERE ");
                    for (int i = 0; i < wheres.length; i++) {
                        String where = wheres[i];
                        sql.append(where);
                        sql.append(i != wheres.length - 1 ? " AND " : "");
                    }
                }
                return executeUpdate(sql.toString());
            }
        } finally {
            destroyed();
        }
        return 0;
    }

    public <T> int update(Class<T> poClass, T po, String... wheres) throws SQLException, ClassNotFoundException {
        try {
            Class aClass = classMap.get(poClass.getName());
            if (aClass == null) {
                System.err.println(poClass.getName() + " not found.");
            } else {
                StringBuffer sql = new StringBuffer();
                sql.append("UPDATE ");
                sql.append(aClass.getSimpleName().toUpperCase());
                sql.append(" SET ");

                //获取字段列表
                List<Field> fs = new ArrayList<>();
                for (Field f : poClass.getDeclaredFields()) {
                    if (f.isAnnotationPresent(Varchar.class)) {
                        f.setAccessible(true);
                        fs.add(f);
                    }
                }
                for (int i = 0; i < fs.size(); i++) {
                    Field f = fs.get(i);
                    sql.append(f.getName());
                    sql.append(" = ");

                    Object value = f.get(po);
                    if (value instanceof String) {
                        sql.append("'");
                        sql.append(value);
                        sql.append("'");
                    } else {
                        sql.append(value);
                    }
                    if (i != fs.size() - 1) {
                        sql.append(",");
                    }
                }

                if (wheres.length > 0) {
                    sql.append(" WHERE ");
                    for (int i = 0; i < wheres.length; i++) {
                        String where = wheres[i];
                        sql.append(where);
                        sql.append(i != wheres.length - 1 ? " AND " : "");
                    }
                }

                System.out.println("SQL ==> " + sql.toString());
                return executeUpdate(sql.toString());
            }
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } finally {
            destroyed();
        }
        return 0;
    }

    public <T> List<T> select(Class<T> poClass, int pageNumber, int pageSize, String... wheres) throws SQLException, ClassNotFoundException {
        ArrayList<T> list = new ArrayList<>();
        try {
            Class aClass = classMap.get(poClass.getName());
            if (aClass == null) {
                System.err.println(poClass.getName() + " not found.");
            } else {
                StringBuffer sql = new StringBuffer();
                sql.append("SELECT ");

                //查询字段
                List<Field> fs = new ArrayList<>();
                for (Field f : poClass.getDeclaredFields()) {
                    if (f.isAnnotationPresent(Varchar.class)) {
                        fs.add(f);
                    }
                }
                for (int i = 0; i < fs.size(); i++) {
                    Field f = fs.get(i);
                    sql.append(f.getName().toUpperCase());
                    if (i != fs.size() - 1) {
                        sql.append(",");
                    }
                }

                sql.append(" FROM ");
                sql.append(aClass.getSimpleName().toUpperCase());

                //条件参数
                if (wheres.length > 0) {
                    sql.append(" WHERE ");
                    for (int i = 0; i < wheres.length; i++) {
                        String where = wheres[i];
                        sql.append(where);
                        sql.append(i != wheres.length - 1 ? " AND " : "");
                    }
                }

                //分页参数
                sql.append(" LIMIT " + (pageNumber - 1) + "," + pageSize);

                System.out.println("SQL ==> " + sql.toString());
                list.addAll(executeQuery(sql.toString(), poClass));
            }
        } finally {
            destroyed();
        }
        return list;
    }

    /**
     * 执行select查询，返回结果列表
     *
     * @param sql     sql select 语句
     * @param poClass 结果集的行数据处理类对象
     * @return
     * @throws SQLException
     * @throws ClassNotFoundException
     */
    public <T> List<T> executeQuery(String sql, Class<T> poClass) throws SQLException, ClassNotFoundException {
        List<T> rsList = new ArrayList<T>();
        try {
            resultSet = getStatement().executeQuery(sql);
            //获取字段列表
            List<Field> fs = new ArrayList<>();
            for (Field f : poClass.getDeclaredFields()) {
                if (f.isAnnotationPresent(Varchar.class)) {
                    f.setAccessible(true);
                    fs.add(f);
                }
            }
            while (resultSet.next()) {
                try {
                    T o = poClass.newInstance();
                    for (int i = 0; i < fs.size(); i++) {
                        Field f = fs.get(i);
                        String string = resultSet.getString(f.getName());
                        f.set(o, string);
                    }
                    rsList.add(o);
                } catch (InstantiationException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
        } finally {
            destroyed();
        }
        return rsList;
    }

    /**
     * 执行sql语句
     *
     * @param sql
     * @throws SQLException
     * @throws ClassNotFoundException
     */
    public void execute(String sql) throws SQLException, ClassNotFoundException {
        try {
            getStatement().execute(sql);
        } finally {
            destroyed();
        }
    }

    /**
     * 执行sql更新语句
     *
     * @param sqls
     * @throws SQLException
     * @throws ClassNotFoundException
     */
    public int executeUpdate(String... sqls) throws SQLException, ClassNotFoundException {
        int count = 0;
        try {
            for (String sql : sqls) {
                count += getStatement().executeUpdate(sql);
            }
        } finally {
            destroyed();
        }
        return count;
    }

    /**
     * 获取数据库连接
     *
     * @return 数据库连接
     * @throws ClassNotFoundException
     * @throws SQLException
     */
    private Connection getConnection() throws ClassNotFoundException, SQLException {
        if (null == connection) {
            Class.forName("org.sqlite.JDBC");
            connection = DriverManager.getConnection("jdbc:sqlite:" + dbFilePath);
        }
        return connection;
    }

    /**
     * 获取数据库Statement对象
     */
    private Statement getStatement() throws SQLException, ClassNotFoundException {
        if (null == statement) {
            statement = getConnection().createStatement();
        }
        return statement;
    }

    /**
     * 数据库资源关闭和释放
     */
    public void destroyed() {
        try {

            if (null != resultSet) {
                resultSet.close();
                resultSet = null;
            }

            if (null != statement) {
                statement.close();
                statement = null;
            }
            if (null != connection) {
                connection.close();
                connection = null;
            }

        } catch (SQLException e) {
            e.printStackTrace();
        }
    }
}